@@ -20,7 +20,6 @@ import os, tempfile, ftplib, http.client
 
 from ._ost_io import *
 from ost import mol, geom, conop, seq
-from ost import LogWarning
 
 class IOProfiles:
   def __init__(self):
@@ -31,11 +30,11 @@ class IOProfiles:
     else:
       processor = conop.HeuristicProcessor()
     self['STRICT'] = IOProfile(dialect='PDB', fault_tolerant=False,
-                               quack_mode=False, processor=processor.Copy())
+                               processor=processor.Copy())
     self['SLOPPY'] = IOProfile(dialect='PDB', fault_tolerant=True,
-                               quack_mode=True, processor=processor.Copy())
+                               processor=processor.Copy())
     self['CHARMM'] = IOProfile(dialect='CHARMM', fault_tolerant=True,
-                               quack_mode=False, processor=processor.Copy())
+                               processor=processor.Copy())
     self['DEFAULT'] = 'STRICT'
 
   def __getitem__(self, key):
@@ -81,7 +80,7 @@ def _override(val1, val2):
     return val1
 
 def LoadPDB(filename, restrict_chains="", no_hetatms=None,
-            fault_tolerant=None, load_multi=False, quack_mode=None,
+            fault_tolerant=None, load_multi=False,
             join_spread_atom_records=None, calpha_only=None,
             profile='DEFAULT', remote=False, remote_repo='pdb',
             dialect=None, seqres=False, bond_feasibility_check=None,
@@ -113,11 +112,6 @@ def LoadPDB(filename, restrict_chains="", no_hetatms=None,
                      multi-PDB files.
   :type load_multi: :class:`bool`
 
-  :param quack_mode: Guess the chemical class for unknown residues based on its
-                     atoms and connectivity. If set, overrides the value of
-                     :attr:`IOProfile.quack_mode`.
-  :type quack_mode: :class:`bool`
-
   :param join_spread_atom_records: If set, overrides the value of 
                                    :attr:`IOProfile.join_spread_atom_records`.
   :type join_spread_atom_records: :class:`bool`
@@ -161,6 +155,8 @@ def LoadPDB(filename, restrict_chains="", no_hetatms=None,
 
   :param seqres: Whether to read SEQRES records. If set to True, the loaded 
                  entity and seqres entry will be returned as a tuple.
+                 If file doesnt contain SEQRES records, the returned
+                 :class:`ost.seq.SequenceList` will be invalid.
   :type seqres: :class:`bool`
 
   :param bond_feasibility_check: Flag for :attr:`IOProfile.processor`. If
@@ -205,7 +201,6 @@ def LoadPDB(filename, restrict_chains="", no_hetatms=None,
   prof.calpha_only=_override(prof.calpha_only, calpha_only)
   prof.no_hetatms=_override(prof.no_hetatms, no_hetatms)
   prof.dialect=_override(prof.dialect, dialect)
-  prof.quack_mode=_override(prof.quack_mode, quack_mode)
   prof.read_conect=_override(prof.read_conect, read_conect)
   if prof.processor:
     prof.processor.check_bond_feasibility=_override(prof.processor.check_bond_feasibility, 
@@ -353,9 +348,10 @@ def LoadCHARMMTraj(crd, dcd_file=None, profile='CHARMM',
 def LoadMMCIF(filename, fault_tolerant=None, calpha_only=None,
               profile='DEFAULT', remote=False, seqres=False, info=False):
   """
-  Load a mmCIF file and return one or more entities. Several options allow to
-  customize the exact behaviour of the mmCIF import. For more information on
-  these options, see :doc:`profile`.
+  Load an mmCIF file and return the first model as an entity.
+
+  Several options allow to customize the exact behaviour of the mmCIF import.
+  For more information on these options, see :doc:`profile`.
   
   Residues are flagged as ligand if they are not waters nor covered by an
   ``entity_poly`` record (ie. they are non-polymer entities in
@@ -435,23 +431,17 @@ def LoadMMCIF(filename, fault_tolerant=None, calpha_only=None,
   try:
     ent = mol.CreateEntity()
     reader = MMCifReader(filename, ent, prof)
-    
+
     # NOTE: to speed up things, we could introduce a restrict_chains parameter
     #       similar to the one in LoadPDB. Here, it would have to be a list/set
     #       of chain-name-strings.
 
     #if reader.HasNext():
-    reader.Parse()
+    reader.Parse() # branch links are connected in here
     if prof.processor:
       prof.processor.Process(ent)
-      reader.info.ConnectBranchLinks()
     #else:
     #  raise IOError("File doesn't contain any entities")
-
-    # Warn about info dependency on seqres
-    if info and not reader.seqres:
-      LogWarning("MMCifInfo is incomplete when seqres=False")
-
     if seqres and info:
       return ent, reader.seqres, reader.info
     if seqres:
@@ -462,6 +452,47 @@ def LoadMMCIF(filename, fault_tolerant=None, calpha_only=None,
   except:
     raise
 
+
+def SaveMMCIF(ent, filename, compound_lib = conop.GetDefaultLib(),
+              data_name="OST_structure", mmcif_conform = True,
+              entity_info = MMCifWriterEntityList()):
+  """
+  Save OpenStructure entity in mmCIF format
+
+  :param ent:           OpenStructure Entity to be saved
+  :param filename:      Filename - .gz suffix triggers gzip compression
+  :param compound_lib:  Compound library required when writing, uses
+                        :func:`ost.conop.GetDefaultLib` if not given
+  :param data_name:     Name of data block that will be written to
+                        mmCIF file. Typically, thats the PDB ID or some
+                        identifier.
+  :param mmcif_conform: Controls processing of structure, i.e. identification
+                        of mmCIF entities etc. before writing. Detailed
+                        description in :ref:`MMCif writing`. In short:
+                        If *mmcif_conform* is set to True, Chains in *ent* are
+                        expected to be valid mmCIF entities with residue numbers
+                        set according to underlying SEQRES. That should be the
+                        case when *ent* has been loaded with :func:`LoadMMCIF`.
+                        If *mmcif_conform* is set to False, heuristics kick in
+                        to identify and separate mmCIF entities based on
+                        :class:`ost.mol.ChemClass` of the residues in a chain.
+  :type ent: :class:`ost.mol.EntityHandle`/:class:`ost.mol.EntityView`
+  :param entity_info: Advanced usage - description in :ref:`MMCif writing`
+  :type filename: :class:`str`
+  :type compound_lib: :class:`ost.conop.CompoundLib`
+  :type data_name: :class:`str`
+  :type mmcif_conform: :class:`bool`
+  :type entity_info: :class:`MMCifWriterEntityList`
+  """
+  if compound_lib is None:
+    raise RuntimeError("Require valid compound library to write mmCIF format")
+  writer = MMCifWriter()
+  writer.SetStructure(ent, compound_lib, mmcif_conform = mmcif_conform,
+                      entity_info = entity_info)
+  writer.Write(data_name, filename)
+
+
+
 # this function uses a dirty trick: should be a member of MMCifInfoBioUnit
 # which is totally C++, but we want the method in Python... so we define it
 # here (__init__) and add it as a member to the class. With this, the first
@@ -525,3 +556,46 @@ def _PDBize(biounit, asu, seqres=None, min_polymer_size=None,
   return pdb_bu
 
 MMCifInfoBioUnit.PDBize = _PDBize
+
+
+def LoadSDF(filename, fault_tolerant=None, profile='DEFAULT'):
+  """
+  Load SDF file from disk and return an entity.
+
+  :param filename: File to be loaded
+  :type filename: :class:`str`
+
+  :param fault_tolerant: Enable/disable fault-tolerant import. If set, overrides
+                         the value of :attr:`IOProfile.fault_tolerant`.
+  :type fault_tolerant: :class:`bool`
+
+  :param profile: Aggregation of flags and algorithms to control import and
+                  processing of molecular structures. Can either be a
+                  :class:`str` specifying one of the default profiles
+                  ['DEFAULT', 'SLOPPY', 'CHARMM', 'STRICT'] or an actual object
+                  of type :class:`ost.io.IOProfile`.
+                  See :doc:`profile` for more info.
+  :type profile: :class:`str`/:class:`ost.io.IOProfile`
+
+  :raises: :exc:`~ost.io.IOException` if the import fails due to an erroneous or
+      inexistent file
+  """
+  def _override(val1, val2):
+    if val2 != None:
+      return val2
+    else:
+      return val1
+
+  if isinstance(profile, str):
+    prof = profiles.Get(profile)
+  elif isinstance(profile, IOProfile):
+    prof = profile.Copy()
+  else:
+    raise TypeError('profile must be of type string or IOProfile, ' + \
+                    'instead of %s' % type(profile))
+  prof.fault_tolerant = _override(prof.fault_tolerant, fault_tolerant)
+
+  reader = SDFReader(filename, prof)
+  ent = mol.CreateEntity()
+  reader.Import(ent)
+  return ent
