@@ -26,6 +26,8 @@ using namespace boost::python;
 #include <ost/io/mol/io_profile.hh>
 #include <ost/io/mol/mmcif_reader.hh>
 #include <ost/io/mol/mmcif_info.hh>
+#include <ost/io/mol/star_writer.hh>
+#include <ost/io/mol/mmcif_writer.hh>
 #include <ost/io/mmcif_str.hh>
 using namespace ost;
 using namespace ost::io;
@@ -55,12 +57,56 @@ boost::python::tuple WrapMMCifStringToEntity(const String& mmcif,
                                    std::get<2>(res));
 }
 
+String WrapEntityToMMCifStringEnt(const ost::mol::EntityHandle& ent,
+                                 const String& data_name,
+                                 ost::conop::CompoundLibPtr compound_lib,
+                                 bool mmcif_conform) {
+  return EntityToMMCifString(ent, data_name, compound_lib,
+                             mmcif_conform);
+}
+
+String WrapEntityToMMCifStringView(const ost::mol::EntityView& ent,
+                                   const String& data_name,
+                                   ost::conop::CompoundLibPtr compound_lib,
+                                   bool mmcif_conform) {
+  return EntityToMMCifString(ent, data_name, compound_lib,
+                             mmcif_conform);
+}
+
+void WrapStarLoopAddData(StarWriterLoop& sl, const boost::python::list& l) {
+  std::vector<StarWriterValue> v;
+  for (int i = 0; i < boost::python::len(l); ++i){
+    v.push_back(boost::python::extract<StarWriterValue>(l[i]));
+  }
+  sl.AddData(v);
+}
+
+void WrapStarWriterWrite(StarWriter& writer, const String& data_name,
+                         const String& filename) {
+  writer.Write(data_name, filename);
+}
+
+void WrapSetStructureHandle(MMCifWriter& writer,
+                            const ost::mol::EntityHandle& ent,
+                            ost::conop::CompoundLibPtr compound_lib,
+                            bool mmcif_conform,
+                            const std::vector<MMCifWriterEntity>& entity_info) {
+  writer.SetStructure(ent, compound_lib, mmcif_conform, entity_info);
+}
+
+void WrapSetStructureView(MMCifWriter& writer,
+                          const ost::mol::EntityView& ent,
+                          ost::conop::CompoundLibPtr compound_lib,
+                          bool mmcif_conform,
+                          const std::vector<MMCifWriterEntity>& entity_info) {
+  writer.SetStructure(ent, compound_lib, mmcif_conform, entity_info);
+}
+
 void export_mmcif_io()
 {
   class_<MMCifReader, boost::noncopyable>("MMCifReader", init<const String&, EntityHandle&, const IOProfile&>())
     .def("Parse", &MMCifReader::Parse)
     .def("SetRestrictChains", &MMCifReader::SetRestrictChains)
-    .def("SetReadCanonicalSeqRes", &MMCifReader::SetReadCanonicalSeqRes)
     .def("GetSeqRes", &MMCifReader::GetSeqRes)
     .def("GetInfo", make_function(&MMCifReader::GetInfo,
                                   return_value_policy<copy_const_reference>()))
@@ -69,12 +115,75 @@ void export_mmcif_io()
                                 return_value_policy<copy_const_reference>()),
                   &MMCifReader::SetRestrictChains)
     .add_property("seqres", &MMCifReader::GetSeqRes)
-    .add_property("read_seqres", &MMCifReader::GetReadSeqRes, 
-                  &MMCifReader::SetReadSeqRes)
     .add_property("info", make_function(&MMCifReader::GetInfo,
                                    return_value_policy<copy_const_reference>()))
     ;
 
+  class_<StarWriterObject, boost::noncopyable>("StarWriterObject", no_init);
+
+  class_<StarWriterValue>("StarWriterValue", no_init)
+    .def("FromInt", &StarWriterValue::FromInt, (arg("int_val"))).staticmethod("FromInt")
+    .def("FromFloat", &StarWriterValue::FromFloat, (arg("float_val"), arg("decimals"))).staticmethod("FromFloat")
+    .def("FromString", &StarWriterValue::FromString, (arg("string_val"))).staticmethod("FromString")
+    .def("GetValue", &StarWriterValue::GetValue, return_value_policy<copy_const_reference>())
+  ;
+
+  class_<StarWriterDataItem, bases<StarWriterObject> >("StarWriterDataItem", init<const String&, const String&, const StarWriterValue&>())
+    .def("GetCategory", &StarWriterDataItem::GetCategory, return_value_policy<copy_const_reference>())
+    .def("GetAttribute", &StarWriterDataItem::GetAttribute, return_value_policy<copy_const_reference>())
+    .def("GetValue", &StarWriterDataItem::GetValue, return_value_policy<copy_const_reference>())
+  ;
+
+  class_<StarWriterLoopDesc, bases<StarWriterObject> >("StarWriterLoopDesc", init<const String&>())
+    .def("GetCategory", &StarWriterLoopDesc::GetCategory, return_value_policy<copy_const_reference>())
+    .def("GetSize", &StarWriterLoopDesc::GetSize)
+    .def("Add", &StarWriterLoopDesc::Add, (arg("attribute")))
+    .def("GetIndex", &StarWriterLoopDesc::GetIndex, (arg("attribute")))
+  ;
+
+  class_<StarWriterLoop, bases<StarWriterObject> >("StarWriterLoop", init<const StarWriterLoopDesc&>())
+    .def("GetDesc", &StarWriterLoop::GetDesc, return_value_policy<reference_existing_object>())
+    .def("GetN", &StarWriterLoop::GetN)
+    .def("AddData", &WrapStarLoopAddData, (arg("data_list")))
+  ;
+
+  class_<StarWriter>("StarWriter", init<>())
+    .def("Push", &StarWriter::Push, arg("star_writer_object"))
+    .def("Write", &WrapStarWriterWrite, (arg("data_name"), arg("filename")))
+  ;
+
+  class_<ChainNameGenerator>("ChainNameGenerator", init<>())
+    .def("Get", &ChainNameGenerator::Get)
+    .def("Reset", &ChainNameGenerator::Reset)
+    ;
+
+  class_<MMCifWriterEntity>("MMCifWriterEntity", no_init)
+    .def("FromPolymer", &MMCifWriterEntity::FromPolymer).staticmethod("FromPolymer")
+    .def("AddHet", &MMCifWriterEntity::AddHet, (arg("rnum"), arg("mon_id")))
+    .add_property("type", &MMCifWriterEntity::type)
+    .add_property("poly_type", &MMCifWriterEntity::poly_type)
+    .add_property("branch_type", &MMCifWriterEntity::branch_type)
+    .add_property("is_poly", &MMCifWriterEntity::is_poly)
+    .add_property("mon_ids", &MMCifWriterEntity::mon_ids)
+    .add_property("seq_olcs", &MMCifWriterEntity::seq_olcs)
+    .add_property("seq_can_olcs", &MMCifWriterEntity::seq_can_olcs)
+    .add_property("asym_ids", &MMCifWriterEntity::asym_ids)
+  ;
+
+  class_<std::vector<MMCifWriterEntity> >("MMCifWriterEntityList", init<>())
+    .def(vector_indexing_suite<std::vector<MMCifWriterEntity> >())
+  ;
+
+  class_<MMCifWriter, bases<StarWriter> >("MMCifWriter", init<>())
+    .def("SetStructure", &WrapSetStructureHandle, (arg("ent"), arg("compound_lib"),
+                                                   arg("mmcif_conform")=true,
+                                                   arg("entity_info")=std::vector<MMCifWriterEntity>()))
+    .def("SetStructure", &WrapSetStructureView, (arg("ent"), arg("compound_lib"),
+                                                 arg("mmcif_conform")=true,
+                                                 arg("entity_info")=std::vector<MMCifWriterEntity>()))
+    .def("GetEntities", &MMCifWriter::GetEntities, return_value_policy<copy_const_reference>())
+  ;
+
   enum_<MMCifInfoCitation::MMCifInfoCType>("MMCifInfoCType")
     .value("Journal", MMCifInfoCitation::JOURNAL)
     .value("Book", MMCifInfoCitation::BOOK)
@@ -190,37 +299,37 @@ void export_mmcif_io()
     .def(vector_indexing_suite<std::vector<MMCifInfoTransOpPtrList >, true >())
   ;
   class_<MMCifInfoStructRef, MMCifInfoStructRefPtr>("MMCifInfoStructRef", no_init)
-  	.add_property("id", make_function(&MMCifInfoStructRef::GetID, 
-  				        return_value_policy<copy_const_reference>()))
-  	.add_property("db_name", make_function(&MMCifInfoStructRef::GetDBName, 
-  				        return_value_policy<copy_const_reference>()))
-  	.add_property("db_id", make_function(&MMCifInfoStructRef::GetDBID, 
-  				        return_value_policy<copy_const_reference>()))
-  	.add_property("entity_id", make_function(&MMCifInfoStructRef::GetEntityID, 
-  				        return_value_policy<copy_const_reference>()))
-  	.add_property("db_access", make_function(&MMCifInfoStructRef::GetDBAccess, 
-  				        return_value_policy<copy_const_reference>()))
-  	.def("GetAlignedSeq", &MMCifInfoStructRef::GetAlignedSeq, arg("align_id"))
-  	.def("GetAlignedSeqs", &MMCifInfoStructRef::GetAlignedSeqs)
-  	.add_property("aligned_seqs", &MMCifInfoStructRef::GetAlignedSeqs)
+    .add_property("id", make_function(&MMCifInfoStructRef::GetID, 
+                  return_value_policy<copy_const_reference>()))
+    .add_property("db_name", make_function(&MMCifInfoStructRef::GetDBName, 
+                  return_value_policy<copy_const_reference>()))
+    .add_property("db_id", make_function(&MMCifInfoStructRef::GetDBID, 
+                  return_value_policy<copy_const_reference>()))
+    .add_property("entity_id", make_function(&MMCifInfoStructRef::GetEntityID, 
+                  return_value_policy<copy_const_reference>()))
+    .add_property("db_access", make_function(&MMCifInfoStructRef::GetDBAccess, 
+                  return_value_policy<copy_const_reference>()))
+    .def("GetAlignedSeq", &MMCifInfoStructRef::GetAlignedSeq, arg("align_id"))
+    .def("GetAlignedSeqs", &MMCifInfoStructRef::GetAlignedSeqs)
+    .add_property("aligned_seqs", &MMCifInfoStructRef::GetAlignedSeqs)
  ; 
   class_<MMCifInfoStructRefSeq, MMCifInfoStructRefSeqPtr>("MMCifInfoStructRefSeq", no_init)
-  	.add_property("align_id", make_function(&MMCifInfoStructRefSeq::GetID, 
-  				        return_value_policy<copy_const_reference>()))
-  	.add_property("chain_name", make_function(&MMCifInfoStructRefSeq::GetChainName, 
-  				        return_value_policy<copy_const_reference>()))
-  	.add_property("seq_begin", &MMCifInfoStructRefSeq::GetSeqBegin)
-  	.add_property("seq_end", &MMCifInfoStructRefSeq::GetSeqEnd)
-  	.add_property("db_begin", &MMCifInfoStructRefSeq::GetDBBegin)
-  	.add_property("db_end", &MMCifInfoStructRefSeq::GetDBEnd)
-  	.add_property("difs", make_function(&MMCifInfoStructRefSeq::GetDifs,
-  				        return_value_policy<copy_const_reference>()))
+    .add_property("align_id", make_function(&MMCifInfoStructRefSeq::GetID, 
+                  return_value_policy<copy_const_reference>()))
+    .add_property("chain_name", make_function(&MMCifInfoStructRefSeq::GetChainName, 
+                  return_value_policy<copy_const_reference>()))
+    .add_property("seq_begin", &MMCifInfoStructRefSeq::GetSeqBegin)
+    .add_property("seq_end", &MMCifInfoStructRefSeq::GetSeqEnd)
+    .add_property("db_begin", &MMCifInfoStructRefSeq::GetDBBegin)
+    .add_property("db_end", &MMCifInfoStructRefSeq::GetDBEnd)
+    .add_property("difs", make_function(&MMCifInfoStructRefSeq::GetDifs,
+                  return_value_policy<copy_const_reference>()))
   ;
   class_<MMCifInfoStructRefSeqDif, 
-  	     MMCifInfoStructRefSeqDifPtr>("MMCifInfoStructRefSeqDif", no_init)
-  	.add_property("details", make_function(&MMCifInfoStructRefSeqDif::GetDetails, 
-  				        return_value_policy<copy_const_reference>()))
-  	.add_property("seq_rnum", &MMCifInfoStructRefSeqDif::GetSeqRNum)
+         MMCifInfoStructRefSeqDifPtr>("MMCifInfoStructRefSeqDif", no_init)
+    .add_property("details", make_function(&MMCifInfoStructRefSeqDif::GetDetails, 
+                  return_value_policy<copy_const_reference>()))
+    .add_property("seq_rnum", &MMCifInfoStructRefSeqDif::GetSeqRNum)
     .add_property("db_rnum", make_function(&MMCifInfoStructRefSeqDif::GetDBRNum,
                                            return_value_policy<copy_const_reference>()))
   ;
@@ -270,13 +379,13 @@ void export_mmcif_io()
   ;
 
   class_<MMCifInfoStructRefs>("MMCifInfoStructRefs", init<>())
-  	.def(vector_indexing_suite<MMCifInfoStructRefs, true>())
+    .def(vector_indexing_suite<MMCifInfoStructRefs, true>())
   ;
   class_<MMCifInfoStructRefSeqs>("MMCifInfoStructRefSeqs", init<>())
-  	.def(vector_indexing_suite<MMCifInfoStructRefSeqs, true>())
+    .def(vector_indexing_suite<MMCifInfoStructRefSeqs, true>())
   ;
   class_<MMCifInfoStructRefSeqDifs>("MMCifInfoStructRefSeqDifs", init<>())
-  	.def(vector_indexing_suite<MMCifInfoStructRefSeqDifs, true>())
+    .def(vector_indexing_suite<MMCifInfoStructRefSeqDifs, true>())
   ;
   class_<std::vector<MMCifInfoBioUnit> >("MMCifInfoBioUnitList", init<>())
     .def(vector_indexing_suite<std::vector<MMCifInfoBioUnit> >())
@@ -359,22 +468,14 @@ void export_mmcif_io()
   ;
 
   class_<MMCifInfoEntityBranchLink>("MMCifInfoEntityBranchLink",
-                                    init<mol::AtomHandle,
-                                    mol::AtomHandle, unsigned char>())
-    .def("GetAtom1", &MMCifInfoEntityBranchLink::GetAtom1)
-    .def("GetAtom2", &MMCifInfoEntityBranchLink::GetAtom2)
-    .def("GetBondOrder", &MMCifInfoEntityBranchLink::GetBondOrder)
-    .def("ConnectBranchLink", &MMCifInfoEntityBranchLink::ConnectBranchLink)
-    .def("SetAtom1", &MMCifInfoEntityBranchLink::SetAtom1)
-    .def("SetAtom2", &MMCifInfoEntityBranchLink::SetAtom2)
-    .def("SetBondOrder", &MMCifInfoEntityBranchLink::SetBondOrder)
-    .def(self_ns::str(self))
-    .add_property("atom1", &MMCifInfoEntityBranchLink::GetAtom1,
-                  &MMCifInfoEntityBranchLink::SetAtom1)
-    .add_property("atom2", &MMCifInfoEntityBranchLink::GetAtom2,
-                  &MMCifInfoEntityBranchLink::SetAtom2)
-    .add_property("bond_order", &MMCifInfoEntityBranchLink::GetBondOrder,
-                  &MMCifInfoEntityBranchLink::SetBondOrder)
+                                    init<int, int, const String&,
+                                         const String&,
+                                         unsigned char>())
+    .add_property("rnum1", &MMCifInfoEntityBranchLink::rnum1)
+    .add_property("rnum2", &MMCifInfoEntityBranchLink::rnum2)
+    .add_property("aname1", &MMCifInfoEntityBranchLink::aname1)
+    .add_property("aname2", &MMCifInfoEntityBranchLink::aname2)
+    .add_property("bond_order", &MMCifInfoEntityBranchLink::bond_order)
   ;
 
   class_<MMCifInfoEntityBranchLinkMap>("MMCifInfoEntityBranchLinkMap", init<>())
@@ -388,6 +489,19 @@ void export_mmcif_io()
     .def(self_ns::str(self))
   ;
 
+  class_<MMCifEntityDesc>("MMCifEntityDesc", init<>())
+   .add_property("type", &MMCifEntityDesc::type)
+   .add_property("entity_type", &MMCifEntityDesc::entity_type)
+   .add_property("entity_poly_type", &MMCifEntityDesc::entity_poly_type)
+   .add_property("branched_type", &MMCifEntityDesc::branched_type)
+   .add_property("details", &MMCifEntityDesc::details)
+   .add_property("seqres_canonical", &MMCifEntityDesc::seqres_canonical)
+   .add_property("seqres_pdbx", &MMCifEntityDesc::seqres_pdbx)
+   .add_property("mon_ids", &MMCifEntityDesc::mon_ids)
+   .add_property("hetero_num", &MMCifEntityDesc::hetero_num)
+   .add_property("hetero_ids", &MMCifEntityDesc::hetero_ids)
+  ;
+
   class_<MMCifInfo>("MMCifInfo", init<>())
     .def("AddCitation", &MMCifInfo::AddCitation)
     .def("GetCitations", make_function(&MMCifInfo::GetCitations,
@@ -399,6 +513,8 @@ void export_mmcif_io()
     .def("GetMethod", &MMCifInfo::GetMethod)
     .def("SetResolution", &MMCifInfo::SetResolution)
     .def("GetResolution", &MMCifInfo::GetResolution)
+    .def("SetEMResolution", &MMCifInfo::SetEMResolution)
+    .def("GetEMResolution", &MMCifInfo::GetEMResolution)
     .def("SetRFree", &MMCifInfo::SetRFree)
     .def("GetRFree", &MMCifInfo::GetRFree)
     .def("SetRWork", &MMCifInfo::SetRWork)
@@ -409,6 +525,10 @@ void export_mmcif_io()
                                    return_value_policy<copy_const_reference>()))
     .def("SetStructDetails", &MMCifInfo::SetStructDetails)
     .def("GetStructDetails", &MMCifInfo::GetStructDetails)
+    .def("SetStructRefs", make_function(&MMCifInfo::SetStructRefs,
+                                   return_value_policy<copy_const_reference>()))
+    .def("GetStructRefs", make_function(&MMCifInfo::GetStructRefs,
+                                   return_value_policy<copy_const_reference>()))
     .def("SetObsoleteInfo", &MMCifInfo::SetObsoleteInfo)
     .def("GetObsoleteInfo", &MMCifInfo::GetObsoleteInfo)
     .def("AddMMCifPDBChainTr", &MMCifInfo::AddMMCifPDBChainTr)
@@ -423,11 +543,12 @@ void export_mmcif_io()
           arg("minor")=-1))
     .def("GetRevisions", &MMCifInfo::GetRevisions)
     .def("AddEntityBranchLink", &MMCifInfo::AddEntityBranchLink)
-    .def("GetEntityBranchLinks", &MMCifInfo::GetEntityBranchLinks)
     .def("GetEntityBranchByChain", &MMCifInfo::GetEntityBranchByChain)
-    .def("ConnectBranchLinks", &MMCifInfo::ConnectBranchLinks)
     .def("GetEntityBranchChainNames", &WrapGetNames)
-    .def("GetEntityBranchChains", &MMCifInfo::GetEntityBranchChains)
+    .def("SetEntityDesc", &MMCifInfo::SetEntityDesc)
+    .def("GetEntityDesc", &MMCifInfo::GetEntityDesc, return_value_policy<copy_const_reference>())
+    .def("GetEntityIds", &MMCifInfo::GetEntityIds)
+    .def("GetEntityIdsOfType", &MMCifInfo::GetEntityIdsOfType)
     .add_property("citations", make_function(&MMCifInfo::GetCitations,
                                    return_value_policy<copy_const_reference>()))
     .add_property("biounits", make_function(&MMCifInfo::GetBioUnits,
@@ -435,6 +556,8 @@ void export_mmcif_io()
     .add_property("method", &MMCifInfo::GetMethod, &MMCifInfo::SetMethod)
     .add_property("resolution", &MMCifInfo::GetResolution,
                   &MMCifInfo::SetResolution)
+    .add_property("em_resolution", &MMCifInfo::GetEMResolution,
+                  &MMCifInfo::SetEMResolution)
     .add_property("r_free", &MMCifInfo::GetRFree, &MMCifInfo::SetRFree)
     .add_property("r_work", &MMCifInfo::GetRWork, &MMCifInfo::SetRWork)
     .add_property("operations", make_function(&MMCifInfo::GetOperations,
@@ -442,7 +565,7 @@ void export_mmcif_io()
     .add_property("struct_details", &MMCifInfo::GetStructDetails,
                   &MMCifInfo::SetStructDetails)
     .add_property("struct_refs", make_function(&MMCifInfo::GetStructRefs,
-    			        return_value_policy<copy_const_reference>()))
+                  return_value_policy<copy_const_reference>()))
     .add_property("obsolete", &MMCifInfo::GetObsoleteInfo,
                   &MMCifInfo::SetObsoleteInfo)
     .add_property("revisions", &MMCifInfo::GetRevisions)
@@ -451,4 +574,14 @@ void export_mmcif_io()
   def("MMCifStrToEntity", &WrapMMCifStringToEntity, (arg("pdb_string"),
                                                      arg("profile")=IOProfile(),
                                                      arg("process")=false));
+
+  def("EntityToMMCifString",  &WrapEntityToMMCifStringEnt, (arg("ent"),
+                                                            arg("data_name"),
+                                                            arg("compound_lib"),
+                                                            arg("mmcif_conform")));
+
+  def("EntityToMMCifString",  &WrapEntityToMMCifStringView, (arg("ent"),
+                                                             arg("data_name"),
+                                                             arg("compound_lib"),
+                                                             arg("mmcif_conform")));
 }
