@@ -31,11 +31,6 @@
 namespace ost { namespace io {
 
 
-bool is_undef(StringRef value)
-{
-	return value.size()==1 && (value[0]=='?' || value[0]=='.');
-}
-
 MMCifReader::MMCifReader(std::istream& stream, mol::EntityHandle& ent_handle,
                          const IOProfile& profile):
   StarParser(stream, true), profile_(profile), ent_handle_(ent_handle)
@@ -59,15 +54,11 @@ void MMCifReader::Init()
   atom_count_           = 0;
   residue_count_        = 0;
   auth_chain_id_        = false;
-  seqres_can_           = false;
   has_model_            = false;
   restrict_chains_      = "";
   subst_res_id_         = "";
   curr_chain_           = mol::ChainHandle();
   curr_residue_         = mol::ResidueHandle();
-  seqres_               = seq::CreateSequenceList();
-  read_seqres_          = true;
-  warned_rule_based_    = false;
   info_                 = MMCifInfo();
 }
 
@@ -80,7 +71,6 @@ void MMCifReader::ClearState()
   atom_count_           = 0;
   category_             = DONT_KNOW;
   warned_name_mismatch_ = false;
-  seqres_               = seq::CreateSequenceList();
   info_                 = MMCifInfo();
   entity_desc_map_.clear();
   authors_map_.clear();
@@ -92,6 +82,7 @@ void MMCifReader::ClearState()
   revision_types_.clear();
   database_PDB_rev_added_ = false;
   entity_branch_link_map_.clear();
+  entity_poly_seq_map_.clear();
 }
 
 void MMCifReader::SetRestrictChains(const String& restrict_chains)
@@ -101,8 +92,20 @@ void MMCifReader::SetRestrictChains(const String& restrict_chains)
 
 bool MMCifReader::OnBeginData(const StringRef& data_name) 
 {
-  LOG_DEBUG("MCIFFReader: " << profile_);
+  LOG_DEBUG("MMCifReader: " << profile_);
   Profile profile_import("MMCifReader::OnBeginData");
+  if (chain_count_ > 0) {
+      std::stringstream ss;
+      ss << "Can only read one data block. Found second one 'data_" << data_name << "'.";
+      if (profile_.fault_tolerant) {
+        LOG_WARNING(ss.str());
+      } else {
+        throw IOException(this->FormatDiagnostic(STAR_DIAG_ERROR,
+                                                 ss.str(),
+                                                 this->GetCurrentLinenum()));
+      }
+
+  }
 
   // IDs in mmCIF files can be any string, so no restrictions here
 
@@ -134,12 +137,13 @@ bool MMCifReader::OnBeginLoop(const StarLoopDesc& header)
     this->TryStoreIdx(CARTN_X, "Cartn_x", header);
     this->TryStoreIdx(CARTN_Y, "Cartn_y", header);
     this->TryStoreIdx(CARTN_Z, "Cartn_z", header);
+    // optional (but warning: mandatory for waters/ligands)
+    indices_[AUTH_SEQ_ID]        = header.GetIndex("auth_seq_id");
+    indices_[PDBX_PDB_INS_CODE]  = header.GetIndex("pdbx_PDB_ins_code");
     // optional
     indices_[OCCUPANCY]          = header.GetIndex("occupancy");
     indices_[B_ISO_OR_EQUIV]     = header.GetIndex("B_iso_or_equiv");
     indices_[GROUP_PDB]          = header.GetIndex("group_PDB");
-    indices_[AUTH_SEQ_ID]        = header.GetIndex("auth_seq_id");
-    indices_[PDBX_PDB_INS_CODE]  = header.GetIndex("pdbx_PDB_ins_code");
     indices_[PDBX_PDB_MODEL_NUM] = header.GetIndex("pdbx_PDB_model_num");
     indices_[FORMAL_CHARGE]     = header.GetIndex("pdbx_formal_charge");
 
@@ -304,27 +308,33 @@ bool MMCifReader::OnBeginLoop(const StarLoopDesc& header)
     cat_available = true;
   } else if (header.GetCategory() == "struct_ref") {
     category_ = STRUCT_REF;
+    // mandatory items
     this->TryStoreIdx(SR_ENTITY_ID, "entity_id", header);
     this->TryStoreIdx(SR_ID, "id", header);
     this->TryStoreIdx(SR_DB_NAME, "db_name", header);
     this->TryStoreIdx(SR_DB_CODE, "db_code", header);
+    // optional items
     indices_[SR_DB_ACCESS]=header.GetIndex("pdbx_db_accession");
     cat_available = true;
   } else if (header.GetCategory() == "struct_ref_seq") {
-    category_ = STRUCT_REF_SEQ;	
+    category_ = STRUCT_REF_SEQ;
+    // mandatory items
     this->TryStoreIdx(SRS_ALIGN_ID, "align_id", header);
     this->TryStoreIdx(SRS_STRUCT_REF_ID, "ref_id", header);
     this->TryStoreIdx(SRS_ENT_ALIGN_BEG, "seq_align_beg", header);
     this->TryStoreIdx(SRS_ENT_ALIGN_END, "seq_align_end", header);
     this->TryStoreIdx(SRS_DB_ALIGN_BEG, "db_align_beg", header);
     this->TryStoreIdx(SRS_DB_ALIGN_END, "db_align_end", header);
+    // optional items
     indices_[SRS_PDBX_STRAND_ID]=header.GetIndex("pdbx_strand_id");
     cat_available = true;
   } else if (header.GetCategory()=="struct_ref_seq_dif") {
     category_ = STRUCT_REF_SEQ_DIF;
+    // mandatory items
     this->TryStoreIdx(SRSD_ALIGN_ID, "align_id", header);
-    this->TryStoreIdx(SRSD_SEQ_RNUM, "seq_num", header);
-    this->TryStoreIdx(SRSD_DB_RNUM, "pdbx_seq_db_seq_num", header);
+    // optional items
+    indices_[SRSD_SEQ_RNUM]=header.GetIndex("seq_num");
+    indices_[SRSD_DB_RNUM]=header.GetIndex("pdbx_seq_db_seq_num");
     indices_[SRSD_DETAILS]=header.GetIndex("details");
     cat_available = true;
   } else if (header.GetCategory()=="database_PDB_rev") {
@@ -388,7 +398,22 @@ bool MMCifReader::OnBeginLoop(const StarLoopDesc& header)
     indices_[BL_ATOM_STEREO_CONFIG_2] = header.GetIndex("atom_stereo_config_2");
     indices_[BL_VALUE_ORDER] = header.GetIndex("value_order");
     cat_available = true;
- }
+ } else if(header.GetCategory() == "entity_poly_seq") {
+  category_ = ENTITY_POLY_SEQ;
+  // mandatory
+  this->TryStoreIdx(EPS_ENTITY_ID, "entity_id", header);
+  this->TryStoreIdx(EPS_MON_ID, "mon_id", header);
+  this->TryStoreIdx(EPS_NUM, "num", header);
+
+  // optional items
+  indices_[EPS_HETERO] = header.GetIndex("hetero");
+  cat_available = true;
+ } else if (header.GetCategory() == "em_3d_reconstruction") {
+    category_ = EM_3D_RECONSTRUCTION;
+    // optional items
+    indices_[EM_RESOLUTION] = header.GetIndex("resolution");
+    cat_available = true;
+  }
   category_counts_[category_]++;
   return cat_available;
 }
@@ -460,15 +485,21 @@ void MMCifReader::ParseAndAddAtom(const std::vector<StringRef>& columns)
   mol::ResNum res_num(0);
   bool valid_res_num = false;
   if (indices_[PDBX_PDB_MODEL_NUM] != -1) {
+    int model_id = TryGetInt(columns[indices_[PDBX_PDB_MODEL_NUM]],
+                             "atom_site.pdbx_PDB_model_num");
     if (has_model_) {
-      if (curr_model_ != TryGetInt(columns[indices_[PDBX_PDB_MODEL_NUM]],
-                                   "atom_site.pdbx_PDB_model_num")) {
+      if (curr_model_ != model_id) {
+        if (warned_ignored_model_.find(model_id) == warned_ignored_model_.end()) {
+          LOG_WARNING("Ignorning new model " << model_id <<
+                      ". Only model " << curr_model_ << " was read.");
+          warned_ignored_model_.insert(model_id);
+        }
         return;
       }
     } else {
+      LOG_INFO("Reading model " << model_id << ".");
       has_model_ = true;
-      curr_model_ = TryGetInt(columns[indices_[PDBX_PDB_MODEL_NUM]],
-      "atom_site.pdbx_PDB_model_num");
+      curr_model_ = model_id;
     }
   }
 
@@ -499,10 +530,13 @@ void MMCifReader::ParseAndAddAtom(const std::vector<StringRef>& columns)
   }
 
   if (indices_[OCCUPANCY] != -1) { // unit test
-    occ = this->TryGetReal(columns[indices_[OCCUPANCY]], "atom_site.occupancy");
+    occ = this->GetRealOrDefault(columns[indices_[OCCUPANCY]],
+                                 "atom_site.occupancy",
+                                 1.0,
+                                 IsUndefined);
   }
   if (indices_[B_ISO_OR_EQUIV] != -1) {
-    if (!is_undef(columns[indices_[B_ISO_OR_EQUIV]])) {
+    if (!IsUndefined(columns[indices_[B_ISO_OR_EQUIV]])) {
       temp = this->TryGetReal(columns[indices_[B_ISO_OR_EQUIV]],
                               "atom_site.B_iso_or_equiv");
     }
@@ -535,9 +569,29 @@ void MMCifReader::ParseAndAddAtom(const std::vector<StringRef>& columns)
 
   if(!curr_residue_) {
     update_residue=true;
-    subst_res_id_ = cif_chain_name +
-                    columns[indices_[AUTH_SEQ_ID]].str() +
-                    columns[indices_[PDBX_PDB_INS_CODE]].str();
+    if (indices_[AUTH_SEQ_ID] != -1 &&
+        indices_[PDBX_PDB_INS_CODE] != -1) {
+      subst_res_id_ = cif_chain_name +
+                      columns[indices_[AUTH_SEQ_ID]].str() +
+                      columns[indices_[PDBX_PDB_INS_CODE]].str();
+    } else if (!valid_res_num) {
+      // Here we didn't have valid residue number in label_seq_id (which is
+      // expected for ligands and waters). To work around that we store the
+      // author residue number information (auth_seq_id + pdbx_PDB_ins_code)
+      // in subst_res_id_. This variable is never read directly, only used
+      // indirectly to detect if we have to create a new residue.
+      // If we're here we had both missing missing value in label_seq_id,
+      // and the auth_seq_id or pdbx_PDB_ins_code were missing.
+      // There may be more elegant ways to detect that we crossed to a new
+      // residue that don't rely on auth_seq_id/pdbx_PDB_ins_code.
+      LOG_WARNING("_atom_site.label_seq_id is invalid for " <<
+        "residue '" << res_name << "' in chain '" << cif_chain_name <<
+        "' and _atom_site.auth_seq_id or _atom_site.pdbx_PDB_ins_code " <<
+        "are missing.");
+      throw IOException(this->FormatDiagnostic(STAR_DIAG_ERROR,
+                                           "Missing residue number information",
+                                               this->GetCurrentLinenum()));
+    }
   } else if (!valid_res_num) {
     if (indices_[AUTH_SEQ_ID] != -1 &&
         indices_[PDBX_PDB_INS_CODE] != -1) {
@@ -552,6 +606,10 @@ void MMCifReader::ParseAndAddAtom(const std::vector<StringRef>& columns)
                         columns[indices_[PDBX_PDB_INS_CODE]].str();
       }
     } else {
+      LOG_WARNING("_atom_site.label_seq_id is invalid for " <<
+        "residue '" << res_name << "' in chain '" << cif_chain_name <<
+        "' and _atom_site.auth_seq_id or _atom_site.pdbx_PDB_ins_code " <<
+        "are missing.");
       throw IOException(this->FormatDiagnostic(STAR_DIAG_ERROR,
                                            "Missing residue number information",
                                                this->GetCurrentLinenum()));
@@ -572,7 +630,7 @@ void MMCifReader::ParseAndAddAtom(const std::vector<StringRef>& columns)
       curr_chain_.SetStringProp("entity_id", ent_id);
       chain_id_pairs_.push_back(std::pair<mol::ChainHandle,String>(curr_chain_,
                                                                    ent_id));
-      info_.AddMMCifEntityIdTr(cif_chain_name, ent_id);
+      info_.AddMMCifEntityIdTr(cif_chain_name, ent_id, profile_.fault_tolerant);
     }
     assert(curr_chain_.IsValid());
   } else if (chain_id_pairs_.back().second != // unit test
@@ -601,8 +659,12 @@ void MMCifReader::ParseAndAddAtom(const std::vector<StringRef>& columns)
         curr_residue_ = editor.AppendResidue(curr_chain_, res_name.str());
       }
       curr_residue_.SetStringProp("pdb_auth_chain_name", auth_chain_name);
-      curr_residue_.SetStringProp("pdb_auth_resnum", columns[indices_[AUTH_SEQ_ID]].str());
-      curr_residue_.SetStringProp("pdb_auth_ins_code", columns[indices_[PDBX_PDB_INS_CODE]].str());
+      if (indices_[AUTH_SEQ_ID] != -1) {
+        curr_residue_.SetStringProp("pdb_auth_resnum", columns[indices_[AUTH_SEQ_ID]].str());
+      }
+      if (indices_[PDBX_PDB_INS_CODE] != -1) {
+        curr_residue_.SetStringProp("pdb_auth_ins_code", columns[indices_[PDBX_PDB_INS_CODE]].str());
+      }
       curr_residue_.SetStringProp("entity_id", columns[indices_[LABEL_ENTITY_ID]].str());
       curr_residue_.SetStringProp("resnum", columns[indices_[LABEL_SEQ_ID]].str());
       warned_name_mismatch_=false;
@@ -628,6 +690,7 @@ void MMCifReader::ParseAndAddAtom(const std::vector<StringRef>& columns)
                     "name. Ignoring atoms for everything but the first");
       } else {
         LOG_WARNING("Residue with number " << res_num 
+                    << " in chain " << curr_chain_.GetName()
                     << " contains a microheterogeneity. Everything but atoms "
                     "for the residue '" << curr_residue_.GetName() 
                     << "' will be ignored");
@@ -655,10 +718,10 @@ void MMCifReader::ParseAndAddAtom(const std::vector<StringRef>& columns)
       }
   } else {
     mol::AtomHandle atom=curr_residue_.FindAtom(aname);
-    if (atom.IsValid() && !profile_.quack_mode) { // unit test
+    if (atom.IsValid()) { // unit test
       if (profile_.fault_tolerant) { // unit test
         LOG_WARNING("duplicate atom '" << aname << "' in residue " 
-                    << curr_residue_);
+                    << curr_residue_ << " only first atom added");
         return;
       }
       throw IOException(this->FormatDiagnostic(STAR_DIAG_ERROR,
@@ -681,15 +744,22 @@ void MMCifReader::ParseAndAddAtom(const std::vector<StringRef>& columns)
                 columns[indices_[GROUP_PDB]][0]=='H');
 }
 
-MMCifReader::MMCifEntityDescMap::iterator MMCifReader::GetEntityDescMapIterator(
+MMCifEntityDescMap::iterator MMCifReader::GetEntityDescMapIterator(
   const String& entity_id)
 {
   MMCifEntityDescMap::iterator edm_it = entity_desc_map_.find(entity_id);
   // if the entity ID is not already stored, insert it with empty values
   if (edm_it == entity_desc_map_.end()) {
     MMCifEntityDesc desc = {.type=mol::CHAINTYPE_N_CHAINTYPES,
+                            .entity_type = "",
+                            .entity_poly_type = "",
+                            .branched_type = "",
                             .details="",
-                            .seqres=""};
+                            .seqres_canonical="",
+                            .seqres_pdbx="",
+                            .mon_ids=std::vector<String>(),
+                            .hetero_num=std::vector<int>(),
+                            .hetero_ids=std::vector<String>()};
     edm_it = entity_desc_map_.insert(entity_desc_map_.begin(),
                                      MMCifEntityDescMap::value_type(entity_id,
                                                                     desc));
@@ -709,6 +779,8 @@ void MMCifReader::ParseEntity(const std::vector<StringRef>& columns)
     if (edm_it->second.type == mol::CHAINTYPE_N_CHAINTYPES) {
       edm_it->second.type = mol::ChainTypeFromString(columns[indices_[E_TYPE]]);
     }
+    // but set entity_type anyways
+    edm_it->second.entity_type = columns[indices_[E_TYPE]].str();
   } else {
     // don't deal with entities without type
     entity_desc_map_.erase(edm_it);
@@ -729,95 +801,20 @@ void MMCifReader::ParseEntityPoly(const std::vector<StringRef>& columns)
   // store type
   if (indices_[EP_TYPE] != -1) {
     edm_it->second.type = mol::ChainTypeFromString(columns[indices_[EP_TYPE]]);
+    edm_it->second.entity_poly_type = columns[indices_[EP_TYPE]].str();
   }
 
-  // store seqres
-  if (edm_it->second.seqres.length() > 0) {
-    throw IOException(this->FormatDiagnostic(STAR_DIAG_ERROR,
-     "entity_poly.pdbx_seq_one_letter_code[_can] clash: sequence for entry '" +
-                                            columns[indices_[ENTITY_ID]].str() +
-                                             "' is already set to '" +
-                                             edm_it->second.seqres + "'.",
-                                             this->GetCurrentLinenum()));
+  // store canonical seqres
+  if (indices_[PDBX_SEQ_ONE_LETTER_CODE_CAN] != -1) {
+    StringRef seqres_can=columns[indices_[PDBX_SEQ_ONE_LETTER_CODE_CAN]];
+    edm_it->second.seqres_canonical = seqres_can.str_no_whitespace();
   }
-  if (read_seqres_) {
-    StringRef seqres;
-    if (seqres_can_) {
-      if (indices_[PDBX_SEQ_ONE_LETTER_CODE_CAN] != -1) {
-        seqres=columns[indices_[PDBX_SEQ_ONE_LETTER_CODE_CAN]];
-        edm_it->second.seqres = seqres.str_no_whitespace();        
-      } else {
-        throw IOException(this->FormatDiagnostic(STAR_DIAG_ERROR,
-                   "'entity_poly.pdbx_seq_one_letter_code_can' not available.'",
-                                                 this->GetCurrentLinenum()));
-      }
-    } else if (indices_[PDBX_SEQ_ONE_LETTER_CODE] != -1) {
-      seqres=columns[indices_[PDBX_SEQ_ONE_LETTER_CODE]];
 
-      conop::CompoundLibBasePtr comp_lib=conop::Conopology::Instance()
-                                                .GetDefaultLib();
-      if (!comp_lib) {
-        if (!warned_rule_based_) {
-          LOG_WARNING("SEQRES import requires a valid compound library to "
-                       "handle non standard compounds. Their One letter "
-                       "codes will be set to X.");      
-        }
-        warned_rule_based_=true;
-        comp_lib = conop::CompoundLibBasePtr(new ost::conop::MinimalCompoundLib);
-      }
-      edm_it->second.seqres = this->ConvertSEQRES(seqres.str_no_whitespace(),
-                                                  comp_lib);
-    } else {
-      throw IOException(this->FormatDiagnostic(STAR_DIAG_ERROR,
-                       "'entity_poly.pdbx_seq_one_letter_code' not available.'",
-                                               this->GetCurrentLinenum()));
+  // store non canonical seqres
+  if (indices_[PDBX_SEQ_ONE_LETTER_CODE] != -1) {
+    StringRef seqres_pdbx=columns[indices_[PDBX_SEQ_ONE_LETTER_CODE]];
+    edm_it->second.seqres_pdbx = seqres_pdbx.str_no_whitespace();
     }
-  }
-}
-
-String MMCifReader::ConvertSEQRES(const String& seqres, 
-                                  conop::CompoundLibBasePtr comp_lib)
-{
-  String can_seqres;
-  for (String::const_iterator i=seqres.begin(), e=seqres.end(); i!=e; ++i) {
-    if (*i=='(') {
-      bool found_end_paren=false;
-      String tlc;
-      tlc.reserve(3);
-      while ((++i)!=seqres.end()) {
-        if (*i==')') {
-          found_end_paren=true;
-          break;
-        }
-        tlc.push_back(*i);
-      }
-      if (!found_end_paren) {
-        throw IOException(this->FormatDiagnostic(STAR_DIAG_ERROR,
-                          "'entity_poly.pdbx_seq_one_letter_code' contains "
-                          "unmatched '('", this->GetCurrentLinenum()));
-      }
-      conop::CompoundPtr compound=comp_lib->FindCompound(tlc, 
-                                                         conop::Compound::PDB);
-      if (!compound) {
-        if (tlc!="UNK") {
-
-          LOG_WARNING("unknown residue '" << tlc << "' in SEQRES record. "
-                      "Setting one-letter-code to 'X'");
-        }
-        can_seqres.push_back('X');
-        continue;
-      }
-      if (compound->GetOneLetterCode()=='?') {
-        can_seqres.push_back('X');
-      } else {
-        can_seqres.push_back(compound->GetOneLetterCode());
-      }
-
-    } else {
-      can_seqres.push_back(*i);
-    }
-  }
-  return can_seqres;
 }
 
 void MMCifReader::ParseCitation(const std::vector<StringRef>& columns)
@@ -888,13 +885,13 @@ void MMCifReader::ParseCitation(const std::vector<StringRef>& columns)
     }
   }
   if (indices_[PDBX_DATABASE_ID_PUBMED] != -1) {
-    if (!is_undef(columns[indices_[PDBX_DATABASE_ID_PUBMED]])) {
+    if (!IsUndefined(columns[indices_[PDBX_DATABASE_ID_PUBMED]])) {
       cit.SetPubMed(this->TryGetInt(columns[indices_[PDBX_DATABASE_ID_PUBMED]],
                                     "citation.pdbx_database_id_PubMed"));
     }
   }
   if (indices_[YEAR] != -1) {
-    if (!is_undef(columns[indices_[YEAR]])) {
+    if (!IsUndefined(columns[indices_[YEAR]])) {
       cit.SetYear(this->TryGetInt(columns[indices_[YEAR]], "citation.year"));
     }
   }
@@ -981,6 +978,14 @@ void MMCifReader::ParseRefine(const std::vector<StringRef>& columns)
   }
 }
 
+void MMCifReader::ParseEm3DReconstruction(const std::vector<StringRef>& columns)
+{
+  StringRef col = columns[indices_[EM_RESOLUTION]];
+  if (col.size()!=1 || (col[0]!='?' && col[0]!='.')) {
+    info_.SetEMResolution(this->TryGetReal(col, "em_3d_reconstruction.resolution"));
+  }
+}
+
 void MMCifReader::ParsePdbxStructAssembly(const std::vector<StringRef>& columns)
 {
   MMCifPSAEntry psa;
@@ -1185,7 +1190,7 @@ void MMCifReader::ParseStruct(const std::vector<StringRef>& columns)
   }
 
   if (indices_[PDBX_FORMULA_WEIGHT] != -1) {
-    if (!is_undef(columns[indices_[PDBX_FORMULA_WEIGHT]])) {
+    if (!IsUndefined(columns[indices_[PDBX_FORMULA_WEIGHT]])) {
       details.SetMass(this->TryGetReal(columns[indices_[PDBX_FORMULA_WEIGHT]],
                                        "struct.pdbx_formula_weight"));
     }
@@ -1604,6 +1609,14 @@ void MMCifReader::OnDataRow(const StarLoopDesc& header,
     LOG_TRACE("processing pdbx_entity_branch_link entry");
     this->ParsePdbxEntityBranchLink(columns);
     break;
+  case ENTITY_POLY_SEQ:
+    LOG_TRACE("processing entity_poly_seq entry");
+    this->ParseEntityPolySeq(columns);
+    break;
+  case EM_3D_RECONSTRUCTION:
+    LOG_TRACE("processing em_3d_reconstruction entry");
+    this->ParseEm3DReconstruction(columns);
+    break;
   default:
     throw IOException(this->FormatDiagnostic(STAR_DIAG_ERROR,
                        "Uncatched category '"+ header.GetCategory() +"' found.",
@@ -1640,17 +1653,17 @@ void MMCifReader::AssignSecStructure(mol::EntityHandle ent)
 
 void MMCifReader::ParseStructRef(const std::vector<StringRef>& columns)
 {
-	String ent_id=columns[indices_[SR_ENTITY_ID]].str();
-	String db_name=columns[indices_[SR_DB_NAME]].str();
-	String db_code=columns[indices_[SR_DB_CODE]].str();
-	String id=columns[indices_[SR_ID]].str();
-	String db_access;
-	if (indices_[SR_DB_ACCESS]!=-1) {
-		db_access=columns[indices_[SR_DB_ACCESS]].str();
-	}
-	MMCifInfoStructRefPtr sr(new MMCifInfoStructRef(id, ent_id, db_name, 
-				                                          db_code, db_access));
-	struct_refs_.push_back(sr);
+  String ent_id=columns[indices_[SR_ENTITY_ID]].str();
+  String db_name=columns[indices_[SR_DB_NAME]].str();
+  String db_code=columns[indices_[SR_DB_CODE]].str();
+  String id=columns[indices_[SR_ID]].str();
+  String db_access;
+  if (indices_[SR_DB_ACCESS]!=-1) {
+    db_access=columns[indices_[SR_DB_ACCESS]].str();
+  }
+  MMCifInfoStructRefPtr sr(new MMCifInfoStructRef(id, ent_id, db_name, 
+                                                  db_code, db_access));
+  struct_refs_.push_back(sr);
 }
 
 void MMCifReader::ParseStructRefSeq(const std::vector<StringRef>& columns)
@@ -1659,43 +1672,43 @@ void MMCifReader::ParseStructRefSeq(const std::vector<StringRef>& columns)
  String sr_id=columns[indices_[SRS_STRUCT_REF_ID]].str();
  String chain_name;
  if (indices_[SRS_PDBX_STRAND_ID]!=-1) {
- 	 chain_name=columns[indices_[SRS_PDBX_STRAND_ID]].str();
+    chain_name=columns[indices_[SRS_PDBX_STRAND_ID]].str();
  }
  std::pair<bool,int> dbbeg=this->TryGetInt(columns[indices_[SRS_DB_ALIGN_BEG]], 
- 		                                        "_struct_ref_seq.db_align_beg",
- 		                                        profile_.fault_tolerant);
+                                             "_struct_ref_seq.db_align_beg",
+                                             profile_.fault_tolerant);
  std::pair<bool,int> dbend=this->TryGetInt(columns[indices_[SRS_DB_ALIGN_END]], 
- 		                                       "_struct_ref_seq.db_align_end",
- 		                                       profile_.fault_tolerant);
+                                            "_struct_ref_seq.db_align_end",
+                                            profile_.fault_tolerant);
  std::pair<bool,int> entbeg=this->TryGetInt(columns[indices_[SRS_ENT_ALIGN_BEG]], 
- 		                                        "_struct_ref_seq.seq_align_beg",
- 		                                        profile_.fault_tolerant);
+                                             "_struct_ref_seq.seq_align_beg",
+                                             profile_.fault_tolerant);
  std::pair<bool,int> entend=this->TryGetInt(columns[indices_[SRS_ENT_ALIGN_END]], 
- 		                                        "_struct_ref_seq.seq_align_END",
- 		                                        profile_.fault_tolerant);
+                                             "_struct_ref_seq.seq_align_END",
+                                             profile_.fault_tolerant);
  if (!(dbbeg.first && dbend.first && entbeg.first && entend.first)) {
- 	 return;
+    return;
  }
  bool found=false;
  for (MMCifInfoStructRefs::iterator i=struct_refs_.begin(), 
- 		  e=struct_refs_.end(); i!=e; ++i) { 
- 	 if ((*i)->GetID()==sr_id) {
-		 (*i)->AddAlignedSeq(aln_id, chain_name, entbeg.second, entend.second, 
-		 		                 dbbeg.second, dbend.second);
-		 found=true;
- 	 	 break;
- 	 }
+       e=struct_refs_.end(); i!=e; ++i) { 
+    if ((*i)->GetID()==sr_id) {
+     (*i)->AddAlignedSeq(aln_id, chain_name, entbeg.second, entend.second, 
+                          dbbeg.second, dbend.second, profile_.fault_tolerant);
+     found=true;
+       break;
+    }
  }
  if (!found) {
- 	 if (profile_.fault_tolerant) {
- 	 	 LOG_ERROR("struct_ref_seq.ref_id points to inexistent struct_ref '"
- 	 	 		       << sr_id <<  "'");
- 	 	 return;
- 	 }
-	 std::stringstream ss;
-	 ss << "struct_ref_seq.ref_id points to inexistent struct_ref '";
-	 ss << sr_id << "'";
-	 throw IOException(ss.str());
+    if (profile_.fault_tolerant) {
+       LOG_ERROR("struct_ref_seq.ref_id points to inexistent struct_ref '"
+                  << sr_id <<  "'");
+       return;
+    }
+   std::stringstream ss;
+   ss << "struct_ref_seq.ref_id points to inexistent struct_ref '";
+   ss << sr_id << "'";
+   throw IOException(ss.str());
  }
 }
 
@@ -1703,42 +1716,48 @@ void MMCifReader::ParseStructRefSeqDif(const std::vector<StringRef>& columns)
 {
   String aln_id=columns[indices_[SRSD_ALIGN_ID]].str();
   String db_rnum;
-  if (!is_undef(columns[indices_[SRSD_DB_RNUM]])) {
+  if (indices_[SRSD_DB_RNUM] != -1) {
     db_rnum=columns[indices_[SRSD_DB_RNUM]].str();
+  } else {
+    db_rnum="?";
+    LOG_INFO("Setting missing struct_ref_seq_dif.pdbx_seq_db_seq_num to '?");
   }
-  std::pair<bool,int> seq_rnum(true, -1);
-  if (!is_undef(columns[indices_[SRSD_SEQ_RNUM]])) {
-    seq_rnum=this->TryGetInt(columns[indices_[SRSD_SEQ_RNUM]],
-                             "_struct_ref_seq_dif.seq_num",
-                             profile_.fault_tolerant);
-    
+
+  std::pair<bool,int> seq_rnum;
+  if (indices_[SRSD_SEQ_RNUM] != -1) {
+    if (!IsUndefined(columns[indices_[SRSD_SEQ_RNUM]])) {
+      seq_rnum=this->TryGetInt(columns[indices_[SRSD_SEQ_RNUM]],
+                               "_struct_ref_seq_dif.seq_num",
+                               profile_.fault_tolerant);
+    }
   }
   if (!seq_rnum.first) {
+    LOG_WARNING("Ignoring struct_ref_seq_dif with missing data item seq_num");
     return;
   }
   String details;
   if (indices_[SRSD_DETAILS]!=-1) {
-	  details=columns[indices_[SRSD_DETAILS]].str();
+    details=columns[indices_[SRSD_DETAILS]].str();
   }
   bool found=false;
   for (MMCifInfoStructRefs::iterator i=struct_refs_.begin(), 
- 		  e=struct_refs_.end(); i!=e; ++i) { 
- 	 if (MMCifInfoStructRefSeqPtr s=(*i)->GetAlignedSeq(aln_id)) {
-		 s->AddDif(seq_rnum.second, db_rnum, details); 
-		 found=true;
- 	 	 break;
- 	 }
+       e=struct_refs_.end(); i!=e; ++i) { 
+    if (MMCifInfoStructRefSeqPtr s=(*i)->GetAlignedSeq(aln_id)) {
+     s->AddDif(seq_rnum.second, db_rnum, details); 
+     found=true;
+       break;
+    }
  }
  if (!found) {
- 	 if (profile_.fault_tolerant) {
- 	 	 LOG_ERROR("struct_ref_seq_dif.align_id points to inexistent "
- 	 	 		       "struct_ref_seq '" << aln_id <<  "'");
- 	 	 return;
- 	 }
-	 std::stringstream ss;
-	 ss << "struct_ref_seq.ref_id points to inexistent struct_ref '";
-	 ss << aln_id << "'";
-	 throw IOException(ss.str());
+    if (profile_.fault_tolerant) {
+       LOG_ERROR("struct_ref_seq_dif.align_id points to inexistent "
+                  "struct_ref_seq '" << aln_id <<  "'");
+       return;
+    }
+   std::stringstream ss;
+   ss << "struct_ref_seq.ref_id points to inexistent struct_ref '";
+   ss << aln_id << "'";
+   throw IOException(ss.str());
  }
 }
 
@@ -1751,6 +1770,7 @@ void MMCifReader::ParsePdbxEntityBranch(const std::vector<StringRef>& columns)
   // store type
   if (indices_[BR_ENTITY_TYPE] != -1) {
     edm_it->second.type = mol::ChainTypeFromString(columns[indices_[EP_TYPE]]);
+    edm_it->second.branched_type = columns[indices_[EP_TYPE]].str();
   }
 }
 
@@ -1804,6 +1824,58 @@ void MMCifReader::ParsePdbxEntityBranchLink(const std::vector<StringRef>& column
   blm_it->second.push_back(link_pair);
 }
 
+void MMCifReader::ParseEntityPolySeq(const std::vector<StringRef>& columns)
+{
+  std::pair<bool, int> tmp = columns[indices_[EPS_NUM]].to_int();
+  if(!tmp.first) {
+    throw IOException(this->FormatDiagnostic(STAR_DIAG_ERROR,
+                                             "Could not cast "
+                                             "_entity_poly_seq.num to integer: "
+                                             + columns[indices_[EPS_NUM]].str(),
+                                             this->GetCurrentLinenum())); 
+  }
+  int num = tmp.second;
+
+  if(num < 1) {
+    throw IOException(this->FormatDiagnostic(STAR_DIAG_ERROR,
+                                             "_entity_poly_seq.num are "
+                                             "expected to be ints >= 1. Got:"
+                                             + columns[indices_[EPS_NUM]].str(),
+                                             this->GetCurrentLinenum())); 
+  }
+
+  // [] inserts new value if not present in container
+  std::map<int, String>& entity_map =
+  entity_poly_seq_map_[columns[indices_[EPS_ENTITY_ID]].str()];
+
+  if(entity_map.find(num) != entity_map.end()) {
+
+    if(indices_[EPS_HETERO] != -1 && columns[indices_[EPS_HETERO]][0] == 'y') {
+      // controlled vocabulary: "y", "yes", "n", "no"
+      // the first two mark hetereogeneous compounds
+
+      // [] inserts new value if not present
+      std::vector<std::pair<int, String> >& hetero_list =
+      entity_poly_seq_h_map_[columns[indices_[EPS_ENTITY_ID]].str()];
+
+      hetero_list.push_back(std::make_pair(num, columns[indices_[EPS_MON_ID]].str()));
+    } else {
+
+      throw IOException(this->FormatDiagnostic(STAR_DIAG_ERROR,
+                                               "_entity_poly_seq.num must be "
+                                               "unique in same "
+                                               "_entity_poly_seq.entity_id. "
+                                               "entity_id: " +
+                                               columns[indices_[EPS_ENTITY_ID]].str() +
+                                               ", num: " +
+                                               columns[indices_[EPS_ENTITY_ID]].str(),
+                                               this->GetCurrentLinenum()));
+    }
+  } else {
+    entity_map[num] = columns[indices_[EPS_MON_ID]].str();
+  }
+}
+
 void MMCifReader::OnEndData()
 {
   mol::XCSEditor editor=ent_handle_.EditXCS(mol::BUFFERED_EDIT);
@@ -1820,15 +1892,17 @@ void MMCifReader::OnEndData()
     if (edm_it != entity_desc_map_.end()) {
       editor.SetChainType(css->first, edm_it->second.type);
       editor.SetChainDescription(css->first, edm_it->second.details);
-      if (edm_it->second.seqres.length() > 0) {
-        seqres_.AddSequence(seq::CreateSequence(css->first.GetName(),
-                                                edm_it->second.seqres));
-        pdb_auth_chain_name = css->first.GetStringProp("pdb_auth_chain_name");
-        info_.AddMMCifPDBChainTr(css->first.GetName(), pdb_auth_chain_name);
-        info_.AddPDBMMCifChainTr(pdb_auth_chain_name, css->first.GetName());
-      } else if (edm_it->second.type!=mol::CHAINTYPE_WATER) {
-        // mark everything that doesn't have SEQRES and isn't of type
-        // water as ligand
+      // Add chain mapping for all chains
+      pdb_auth_chain_name = css->first.GetStringProp("pdb_auth_chain_name");
+      info_.AddMMCifPDBChainTr(css->first.GetName(), pdb_auth_chain_name,
+                               profile_.fault_tolerant);
+
+      if (edm_it->second.entity_type=="polymer") {
+        // PDB -> mmCIF chain mapping only for polymers
+        // This is not a 1:1 mapping because of ligands
+        info_.AddPDBMMCifChainTr(pdb_auth_chain_name, css->first.GetName(),
+                                 profile_.fault_tolerant);
+      } else if (edm_it->second.entity_type=="non-polymer") {
         mol::ChainHandle chain=css->first;
         mol::ResidueHandleList residues=chain.GetResidueList();
         for (mol::ResidueHandleList::iterator 
@@ -1838,7 +1912,8 @@ void MMCifReader::OnEndData()
       }
     } else {
       LOG_WARNING("No entity description found for atom_site.label_entity_id '"
-                  << css->second << "'");
+                  << css->second << "'. SEQRES, chain mapping and ligand "
+                  << " annotation will be missing.");
     }
     // find
     blm_it = entity_branch_link_map_.find(css->second);
@@ -1846,14 +1921,23 @@ void MMCifReader::OnEndData()
     if (blm_it != entity_branch_link_map_.end()) {
       for (bl_it = blm_it->second.begin(); bl_it != blm_it->second.end();
            ++bl_it) {
+        info_.AddEntityBranchLink(css->second,
+                                  bl_it->res_num_1, bl_it->res_num_2,
+                                  bl_it->atm_nm_1, bl_it->atm_nm_2,
+                                  bl_it->bond_order);
+
+        // and directly connect if respective atoms are available...
         mol::ResidueHandle res1 = css->first.FindResidue(to_res_num(
                                                          bl_it->res_num_1, ' '));
         mol::ResidueHandle res2 = css->first.FindResidue(to_res_num(
                                                          bl_it->res_num_2, ' '));
-        info_.AddEntityBranchLink(css->first.GetName(),
-                                  res1.FindAtom(bl_it->atm_nm_1),
-                                  res2.FindAtom(bl_it->atm_nm_2),
-                                  bl_it->bond_order);
+        if(res1.IsValid() && res2.IsValid()) {
+          mol::AtomHandle a1 = res1.FindAtom(bl_it->atm_nm_1);
+          mol::AtomHandle a2 = res2.FindAtom(bl_it->atm_nm_2);
+          if(a1.IsValid() && a2.IsValid()) {
+            editor.Connect(a1, a2, bl_it->bond_order);
+          }
+        }
       }
     }
   }
@@ -1943,6 +2027,27 @@ void MMCifReader::OnEndData()
     }
   }
 
+  // conclude EntityDesc (add entity_poly_seq if present) and add to MMCifInfo
+  for(auto &entity_it: entity_desc_map_) {
+    if(entity_poly_seq_map_.find(entity_it.first) != entity_poly_seq_map_.end()) {
+      int max_num = 1;
+      for(auto &seqres_it: entity_poly_seq_map_[entity_it.first]) {
+        max_num = std::max(max_num, seqres_it.first);
+      }
+      entity_it.second.mon_ids.assign(max_num, "?");
+      for(auto &seqres_it: entity_poly_seq_map_[entity_it.first]) {
+        entity_it.second.mon_ids[seqres_it.first-1] = seqres_it.second;
+      }
+    }
+    if(entity_poly_seq_h_map_.find(entity_it.first) != entity_poly_seq_h_map_.end()) {
+      for(auto &hetero_it: entity_poly_seq_h_map_[entity_it.first]) {
+        entity_it.second.hetero_num.push_back(hetero_it.first);
+        entity_it.second.hetero_ids.push_back(hetero_it.second);
+      }
+    }
+    info_.SetEntityDesc(entity_it.first, entity_it.second);
+  }
+
   LOG_INFO("imported "
            << chain_count_ << " chains, "
            << residue_count_ << " residues, "
@@ -2011,4 +2116,68 @@ String OSTBondOrderToMMCifValueOrder(const unsigned char bond_order)
   return String("");
 }
 
+seq::SequenceList MMCifReader::GetSeqRes() const {
+  std::map<String, String> entity_seqres_map; // Map entity_id -> seqres
+  seq::SequenceList seqres_list = seq::CreateSequenceList();
+
+  // We need a compound lib for the conversion
+  conop::CompoundLibBasePtr comp_lib=conop::Conopology::Instance()
+                                            .GetDefaultLib();
+  if (!comp_lib) {
+    LOG_WARNING("SEQRES requires a valid compound library to "
+                 "handle non standard compounds. Their One letter "
+                 "codes will be set to X.");
+    comp_lib = conop::CompoundLibBasePtr(new ost::conop::MinimalCompoundLib);
+  }
+
+  // Generate the SEQRES for every entity
+  for(auto const &entity_it: entity_desc_map_) {
+    if (entity_it.second.entity_type == "polymer") {
+      entity_seqres_map[entity_it.first] = "";
+      auto mon_ids = entity_it.second.mon_ids;
+//      if (mon_ids.size() == 0) {
+//        // We hit this if there was an _entity_poly category but no _entity_poly_seq
+//        LOG_WARNING("No SEQRES found for entity '"
+//                    << entity_it.first << "'.");
+//      }
+      entity_seqres_map[entity_it.first].reserve(mon_ids.size());
+      for (auto const &mon_id: mon_ids) {
+        conop::CompoundPtr compound=comp_lib->FindCompound(mon_id, conop::Compound::PDB);
+        if (!compound) {
+          if (mon_id != "UNK") {
+            LOG_WARNING("unknown residue '" << mon_id << "' in SEQRES record. "
+                        "Setting one-letter-code to 'X'");
+          }
+          entity_seqres_map[entity_it.first].push_back('X');
+          continue;
+        }
+        if (compound->GetOneLetterCode()=='?') {
+          entity_seqres_map[entity_it.first].push_back('X');
+        } else {
+          entity_seqres_map[entity_it.first].push_back(compound->GetOneLetterCode());
+        }
+      }
+    }
+  }
+
+  // Assign
+  for (auto const &css: chain_id_pairs_) {
+    auto entity_seqres_map_it = entity_seqres_map.find(css.second);
+    if (entity_seqres_map_it != entity_seqres_map.end()) {
+      if (entity_seqres_map_it->second == "") {
+        // We hit this if there was an _entity_poly category but no _entity_poly_seq
+        LOG_WARNING("No SEQRES found for chain '"
+                    << css.first << "'. Most likely the entity_poly_seq "
+                    << "category was missing from the input file.");
+      } else {
+        seqres_list.AddSequence(seq::CreateSequence(css.first.GetName(),
+                                                    entity_seqres_map_it->second));
+      }
+    }
+    // else: either non polymer chain, or no entity_poly was available (this
+    // triggered a warning before)
+  }
+  return seqres_list;
+}
+
 }}
