@@ -92,7 +92,9 @@ void PDBReader::Init(const boost::filesystem::path& loc)
     in_.push(boost::iostreams::gzip_decompressor());
   }
   in_.push(instream_);
-  if(!infile_) throw IOException("could not open "+loc.string());
+  if(!infile_) throw IOException("[Errno " + std::to_string(errno) + "] " +
+                     std::string(strerror(errno)) +
+                     ": '" + loc.string() + "'");
   line_num_=0;
   if(boost::iequals(boost::filesystem::extension(loc), ".pqr")) {
     is_pqr_=true;
@@ -195,7 +197,7 @@ void PDBReader::ParseCompndEntry (const StringRef& line, int line_num)
   } 
       //currently only these are parsed
   if (!(key.str()=="MOL_ID")&&!(key.str()=="CHAIN")){
-    LOG_INFO("reading COMPND record on line " << line_num<< "is not supported");
+    LOG_INFO("reading COMPND record on line " << line_num<< " is not supported");
     if (data_continues_) {
       skip_next_=true;
     } else {
@@ -737,10 +739,15 @@ void PDBReader::ParseAndAddAtom(const StringRef& line, int line_num,
       if(charge.first) {
         if(line[79] != '-' && line[79] != '+') {
           std::stringstream ss;
-          ss << "error on line " << line_num << ": "
-             << "expect charge in format 1+, 2-, etc. got: "
+          ss << "invalid charge on line " << line_num << ": "
+             << "expected 1+, 2-, etc. got: "
              << line.substr(78, 2);
-          throw IOException(ss.str());      
+          if (profile_.fault_tolerant) {
+            LOG_WARNING(ss.str());
+            charge.first = 0.0;
+          } else {
+            throw IOException(ss.str());
+          }
         }
         if(line[79] == '-') charge.second *= (-1);
       }
@@ -837,26 +844,24 @@ void PDBReader::ParseAndAddAtom(const StringRef& line, int line_num,
          << "residue with number " << res_num << " has more than one name.";
       throw IOException(ss.str());
     }
-    if(!profile_.quack_mode) {
-      if (!warned_name_mismatch_) {
-        if (alt_loc==' ') {
-          LOG_WARNING("Residue with number " << res_num << " has more than one name. "
-                      "Ignoring atoms for everything but the first");        
-        } else {
-          LOG_WARNING("Residue with number " << res_num 
-                      << " contains a microheterogeneity. Everything but atoms for "
-                      << "the residue '" << curr_residue_.GetName() 
-                      << "' will be ignored");
-        }
+    if (!warned_name_mismatch_) {
+      if (alt_loc==' ') {
+        LOG_WARNING("Residue with number " << res_num << " has more than one name. "
+                    "Ignoring atoms for everything but the first");        
+      } else {
+        LOG_WARNING("Residue with number " << res_num 
+                    << " contains a microheterogeneity. Everything but atoms for "
+                    << "the residue '" << curr_residue_.GetName() 
+                    << "' will be ignored");
       }
-      warned_name_mismatch_=true;
-      return;
     }
+    warned_name_mismatch_=true;
+    return;
   }
   Real b=temp.first ? temp.second : 0.0;
   Real o=occ.first ? occ.second : 1.0;
 
-  if (!profile_.quack_mode && alt_loc!=' ') {
+  if (alt_loc!=' ') {
     // Check if there is already a atom with the same name.
     mol::AtomHandle me=curr_residue_.FindAtom(aname);
     if (me.IsValid()) {
@@ -864,7 +869,7 @@ void PDBReader::ParseAndAddAtom(const StringRef& line, int line_num,
         editor.AddAltAtomPos(String(1, alt_loc), me, apos, o, b);
       } catch (Error&) {
         LOG_INFO("Ignoring atom alt location since there is already an atom "
-                     "with name " << aname << ", but without an alt loc");
+                 "with name " << aname << ", but without an alt loc");
         return;
       }
       return;
@@ -875,10 +880,10 @@ void PDBReader::ParseAndAddAtom(const StringRef& line, int line_num,
     }
   } else {
     mol::AtomHandle atom=curr_residue_.FindAtom(aname);
-    if (atom.IsValid() && !profile_.quack_mode) {
+    if (atom.IsValid()) {
       if (profile_.fault_tolerant) {
         LOG_WARNING("duplicate atom '" << aname << "' in residue " 
-                    << curr_residue_);
+                    << curr_residue_ << " only first atom added");
         return;
       }
       throw IOException("duplicate atom '"+aname+"' in residue "+
