@@ -1,3 +1,13 @@
+"""
+.. note::
+
+  This is a new implementation of the stereochemistry checks, introduced in
+  OpenStructure 2.4, with support for nucleotides. The
+  :doc:`previous stereochemistry checks <stereochemistry_deprecated>` that come
+  with `Mariani et al. <https://dx.doi.org/10.1093/bioinformatics/btt473>`_ are
+  considered deprecated.
+"""
+
 import os
 import json
 import datetime
@@ -365,7 +375,7 @@ class ClashInfo:
         self.dist = dist
         self.tolerated_dist = tolerated_dist
 
-    def ToJSON(self):
+    def ToJSON(self, decimals = 3):
         """ Return JSON serializable dict
 
         Atoms are represented by a string in format:
@@ -373,8 +383,8 @@ class ClashInfo:
         """
         return {"a1": _AtomToQualifiedName(self.a1),
                 "a2": _AtomToQualifiedName(self.a2),
-                "dist": self.dist,
-                "tolerated_dist": self.tolerated_dist}
+                "dist": round(self.dist, decimals),
+                "tolerated_dist": round(self.tolerated_dist, decimals)}
 
 
 class BondViolationInfo:
@@ -395,7 +405,7 @@ class BondViolationInfo:
         self.exp_length = exp_length
         self.std = std
 
-    def ToJSON(self):
+    def ToJSON(self, decimals = 3):
         """ Return JSON serializable dict
 
         Atoms are represented by a string in format:
@@ -403,9 +413,9 @@ class BondViolationInfo:
         """
         return {"a1": _AtomToQualifiedName(self.a1),
                 "a2": _AtomToQualifiedName(self.a2),
-                "length": self.length,
-                "exp_length": self.exp_length,
-                "std": self.std}
+                "length": round(self.length, decimals),
+                "exp_length": round(self.exp_length, decimals),
+                "std": round(self.std, decimals)}
 
 
 class AngleViolationInfo:
@@ -428,7 +438,7 @@ class AngleViolationInfo:
         self.exp_angle = exp_angle
         self.std = std
 
-    def ToJSON(self):
+    def ToJSON(self, decimals = 3):
         """ Return JSON serializable dict
 
         Atoms are represented by a string in format:
@@ -437,9 +447,9 @@ class AngleViolationInfo:
         return {"a1": _AtomToQualifiedName(self.a1),
                 "a2": _AtomToQualifiedName(self.a2),
                 "a3": _AtomToQualifiedName(self.a3),
-                "angle": self.angle,
-                "exp_angle": self.exp_angle,
-                "std": self.std}
+                "angle": round(self.angle, decimals),
+                "exp_angle": round(self.exp_angle, decimals),
+                "std": round(self.std, decimals)}
 
 
 def GetClashes(ent, vdw_radii = None, tolerance = 1.5, disulfid_dist = 2.03,
